@@ -85,6 +85,9 @@ extern Bool bInitAST1180(ScrnInfoPtr pScrn);
 extern void GetAST1180DRAMInfo(ScrnInfoPtr pScrn);
 extern void vEnableASTVGAMMIO(ScrnInfoPtr pScrn);
 
+extern Bool ReadEDID_M68K(ScrnInfoPtr pScrn, BYTE *pEDIDData);
+extern UCHAR GetLinkMaxCLK(ScrnInfoPtr pScrn);
+
 extern Bool bInitCMDQInfo(ScrnInfoPtr pScrn, ASTRecPtr pAST);
 extern Bool bEnableCMDQ(ScrnInfoPtr pScrn, ASTRecPtr pAST);
 extern void vDisable2D(ScrnInfoPtr pScrn, ASTRecPtr pAST);
@@ -291,7 +294,10 @@ ASTProbe(DriverPtr drv, int flags)
 				   devSections, numDevSections,
 				   drv, &usedChips);
 
-    free(devSections);
+    if (numUsed <= 0) {
+	free(devSections);
+	return FALSE;
+    }
 
     if (flags & PROBE_DETECT) {
         if (numUsed > 0)
@@ -309,7 +315,9 @@ ASTProbe(DriverPtr drv, int flags)
                                "ast: The PCI device 0x%x at %2.2d@%2.2d:%2.2d:%1.1d has a kernel module claiming it.\n",
                                pPci->device_id, pPci->bus, pPci->domain, pPci->dev, pPci->func);
                     xf86DrvMsg(0, X_ERROR,
-                               "cirrus: This driver cannot operate until it has been unloaded.\n");
+                               "ast: This driver cannot operate until it has been unloaded.\n");
+                    xf86UnclaimPciSlot(pPci, devSections[0]);
+                    free(devSections);
                     return FALSE;
                 }
             }
@@ -343,6 +351,7 @@ ASTProbe(DriverPtr drv, int flags)
         }  /* end of for-loop */
     } /* end of if flags */
 
+    free(devSections);
     free(usedChips);
 
     return foundScreen;
@@ -657,7 +666,9 @@ ASTPreInit(ScrnInfoPtr pScrn, int flags)
        bASTRegInit(pScrn);
 
        /* Get Chip Type */
-       if (PCI_DEV_REVISION(pAST->PciInfo) >= 0x20)
+       if (PCI_DEV_REVISION(pAST->PciInfo) >= 0x30)
+           pAST->jChipType = AST2400;
+       else if (PCI_DEV_REVISION(pAST->PciInfo) >= 0x20)
            pAST->jChipType = AST2300;
        else if (PCI_DEV_REVISION(pAST->PciInfo) >= 0x10)
            GetChipType(pScrn);
@@ -707,7 +718,7 @@ ASTPreInit(ScrnInfoPtr pScrn, int flags)
    clockRanges->doubleScanAllowed = FALSE;
 
    /* Add for AST2100, ycchen@061807 */
-   if ((pAST->jChipType == AST2100) || (pAST->jChipType == AST2200) || (pAST->jChipType == AST2300) || (pAST->jChipType == AST1180))
+   if ((pAST->jChipType == AST2100) || (pAST->jChipType == AST2200) || (pAST->jChipType == AST2300) || (pAST->jChipType == AST2400) || (pAST->jChipType == AST1180))
    {
        maxPitch  = 1920;
        maxHeight = 1200;
@@ -1023,7 +1034,7 @@ ASTScreenInit(SCREEN_INIT_ARGS_DECL)
    xf86DPMSInit(pScreen, ASTDisplayPowerManagementSet, 0);
 
 #ifdef AstVideo
-   if ( (pAST->jChipType == AST1180) || (pAST->jChipType == AST2300) )
+   if ( (pAST->jChipType == AST1180) || (pAST->jChipType == AST2300) || (pAST->jChipType == AST2400) )
    {
        xf86DrvMsg(pScrn->scrnIndex, X_INFO,"AST Initial Video()\n");
        ASTInitVideo(pScreen);
@@ -1217,7 +1228,7 @@ ASTValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
       if ( (mode->CrtcHDisplay == 1600) && (mode->CrtcVDisplay == 900) )
           return MODE_OK;
 
-      if ( (pAST->jChipType == AST2100) || (pAST->jChipType == AST2200) || (pAST->jChipType == AST2300) || (pAST->jChipType == AST1180) )
+      if ( (pAST->jChipType == AST2100) || (pAST->jChipType == AST2200) || (pAST->jChipType == AST2300) || (pAST->jChipType == AST2400) || (pAST->jChipType == AST1180) )
       {
           if ( (mode->CrtcHDisplay == 1920) && (mode->CrtcVDisplay == 1080) )
               return MODE_OK;
@@ -1278,10 +1289,14 @@ ASTGetRec(ScrnInfoPtr pScrn)
 static void
 ASTFreeRec(ScrnInfoPtr pScrn)
 {
+   ASTRecPtr pAST = ASTPTR(pScrn);
+
    if (!pScrn)
       return;
    if (!pScrn->driverPrivate)
       return;
+   if (pAST->pDP501FWBufferVirtualAddress)
+       free(pAST->pDP501FWBufferVirtualAddress);
    free(pScrn->driverPrivate);
    pScrn->driverPrivate = 0;
 }
@@ -1546,6 +1561,12 @@ ASTProbeDDC(ScrnInfoPtr pScrn, int index)
    {
       if (pAST->jChipType == AST1180)
           Flags = GetVGA2EDID(pScrn, DDC_data);
+      else if (pAST->jTxChipType == Tx_DP501)
+      {
+          Flags = ReadEDID_M68K(pScrn, DDC_data);
+          if (Flags == FALSE)
+              Flags = GetVGAEDID(pScrn, DDC_data);
+      }
       else
           Flags = GetVGAEDID(pScrn, DDC_data);
 
@@ -1590,6 +1611,14 @@ ASTDoDDC(ScrnInfoPtr pScrn, int index)
    {
       if (pAST->jChipType == AST1180)
           Flags = GetVGA2EDID(pScrn, DDC_data);
+      else if (pAST->jTxChipType == Tx_DP501)
+      {
+	      pAST->DP501_MaxVCLK = 0xFF;
+          Flags = ReadEDID_M68K(pScrn, DDC_data);
+          if (Flags) pAST->DP501_MaxVCLK = GetLinkMaxCLK(pScrn);
+          else
+              Flags = GetVGAEDID(pScrn, DDC_data);
+      }
       else
           Flags = GetVGAEDID(pScrn, DDC_data);
 
